// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;

/**
 * <p>
 * MinMaxTextSection class.
 * </p>
 *
 * @author c.mathew
 * @created 23 Jul 2013
 * @version 1.0
 */
public class MinMaxTextSection extends AbstractFormSection<DerivedUnitFacade> {

	private final TextWithLabelElement text_freeText;
	private final NumberWithLabelElement text_minVal;
	private final NumberWithLabelElement text_maxVal;
	private int cursorPosition;

	// unit types handled by this section
	public enum UnitType {
		ELEVATION,
		DIST_TO_GROUND,
		DIST_TO_WATER
	}

	private UnitType unitType = UnitType.ELEVATION;

	/**
	 * <p>
	 * Constructor for DateDetailSection.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param style
	 *            a int.
	 */
	protected MinMaxTextSection(CdmFormFactory formFactory,
			ICdmFormElement parentElement, UnitType unitType, int style) {
		super(formFactory, parentElement, style);
		this.unitType = unitType;
		getLayoutComposite().setLayout(LayoutConstants.LAYOUT(2, false));

		switch(unitType) {
			case ELEVATION:
				text_minVal = formFactory.createNumberTextWithLabelElement(this, "Min [m] : ", getMinimum(), style);
				text_maxVal = formFactory.createNumberTextWithLabelElement(this, "Max [m] : ", getMaximum(), style);
				break;
			default:
				text_minVal = formFactory.createNumberTextWithLabelElement(this, "Min [m] : ", getMinimum(), style);
				text_maxVal = formFactory.createNumberTextWithLabelElement(this, "Max [m] : ", getMaximum(), style);
				break;
		}
		text_freeText = formFactory.createTextWithLabelElement(this,
				"Freetext : ", getFreetext(), style);

		formFactory.addPropertyChangeListener(this);
	}

	/**
	 * <p>
	 * Setter for the field <code>timePeriod</code>.
	 * </p>
	 *
	 * @param timePeriod
	 *            a {@link eu.etaxonomy.cdm.model.common.TimePeriod} object.
	 */
	@Override
	public void setEntity(DerivedUnitFacade gatheringEvent) {
		super.setEntity(gatheringEvent);
		updateValues();
		updateTitle();
	}

	private void updateValues() {
		if(getEntity() != null) {
			switch(unitType) {
			case ELEVATION:
				if(getEntity().getAbsoluteElevation() != null) {
					text_minVal.setNumber(getEntity().getAbsoluteElevation());
				}
				if(getEntity().getAbsoluteElevationMaximum() != null) {
					text_maxVal.setNumber(getEntity().getAbsoluteElevationMaximum());
				}
				if(getEntity().getAbsoluteElevationText() != null) {
					text_freeText.setText(getEntity().getAbsoluteElevationText());
				}
				break;
			case DIST_TO_GROUND:
				if(getEntity().getDistanceToGround() != null) {
					text_minVal.setNumber(getEntity().getDistanceToGround());
				}
				if(getEntity().getDistanceToGroundMax() != null) {
					text_maxVal.setNumber(getEntity().getDistanceToGroundMax());
				}
				if(getEntity().getDistanceToGroundText() != null) {
					text_freeText.setText(getEntity().getDistanceToGroundText());
				}
				break;
			case DIST_TO_WATER:
				if(getEntity().getDistanceToWaterSurface() != null) {
					text_minVal.setNumber(getEntity().getDistanceToWaterSurface());
				}
				if(getEntity().getDistanceToWaterSurfaceMax() != null) {
					text_maxVal.setNumber(getEntity().getDistanceToWaterSurfaceMax());
				}
				if(getEntity().getDistanceToWaterSurfaceText() != null) {
					text_freeText.setText(getEntity().getDistanceToWaterSurfaceText());
				}
				break;
			default:
				break;
			}

		}
	}
	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();

		if (getElements().contains(eventSource)) {
			if (event instanceof CdmPropertyChangeEvent) {
				if (((CdmPropertyChangeEvent) event).hasException()) {
					handleException((CdmPropertyChangeEvent) event);
					return;
				}
			}
			handleEvent(eventSource);
		}
	}

	/**
	 * @param event
	 */
	private void handleException(CdmPropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this,
				event.getException()));
	}

	private void handleEvent(Object eventSource) {
		if (eventSource == text_minVal) {
			updateMinimum();
		} else if (eventSource == text_maxVal) {
			updateMaximum();
		} else if (eventSource == text_freeText) {
			updateFreetext();
		}
        updateTitle();
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

    private void updateTitle(){
        String title = "";
        if(text_minVal.getText()!=null){
            title += text_minVal.getText();
        }
        if(text_maxVal.getText()!=null && !text_maxVal.getText().equals("")){
            if(!title.equals("")){
                title += " - "+text_maxVal.getText();
            }
        }
        if(title.equals("") && text_freeText.getText()!=null){
            title = text_freeText.getText();
        }
        this.setText(title);
        layout();
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.AbstractFormSection#dispose()
	 */
	/** {@inheritDoc} */
	@Override
	public void dispose() {
		formFactory.removePropertyChangeListener(this);
		super.dispose();
	}

	private void updateMinimum() {
		switch(unitType) {
		case ELEVATION:
			getEntity().setAbsoluteElevation(text_minVal.getDouble()!=null?text_minVal.getDouble().intValue():null);
			break;
		case DIST_TO_GROUND:
			getEntity().setDistanceToGround(text_minVal.getDouble());
			break;
		case DIST_TO_WATER:
			getEntity().setDistanceToWaterSurface(text_minVal.getDouble());
			break;
		}
	}

	private Number getMinimum() {
		if(getEntity() == null) {
			return null;
		}
		switch(unitType) {
		case ELEVATION:
			return getEntity().getAbsoluteElevation();
		case DIST_TO_GROUND:
			return getEntity().getDistanceToGround();
		case DIST_TO_WATER:
			return getEntity().getDistanceToWaterSurface();
		}
		return null;
	}

	private void updateMaximum() {
		switch(unitType) {
		case ELEVATION:
			getEntity().setAbsoluteElevationMax(text_maxVal.getDouble()!=null?text_maxVal.getDouble().intValue():null);
			break;
		case DIST_TO_GROUND:
			getEntity().setDistanceToGroundMax(text_maxVal.getDouble());
			break;
		case DIST_TO_WATER:
			getEntity().setDistanceToWaterSurfaceMax(text_maxVal.getDouble());
			break;
		}
	}

	private Number getMaximum() {
		if(getEntity() == null) {
			return null;
		}
		switch(unitType) {
		case ELEVATION:
			return getEntity().getAbsoluteElevationMaximum();
		case DIST_TO_GROUND:
			return getEntity().getDistanceToGroundMax();
		case DIST_TO_WATER:
			return getEntity().getDistanceToWaterSurfaceMax();
		}
		return null;
	}

	private void updateFreetext() {
		switch(unitType) {
		case ELEVATION:
			getEntity().setAbsoluteElevationText(text_freeText.getText());
			break;
		case DIST_TO_GROUND:
			getEntity().setDistanceToGroundText(text_freeText.getText());
			break;
		case DIST_TO_WATER:
			getEntity().setDistanceToWaterSurfaceText(text_freeText.getText());
			break;
		}
	}

	private String getFreetext() {
		if(getEntity() == null) {
			return null;
		}
		switch(unitType) {
		case ELEVATION:
			return getEntity().getAbsoluteElevationText();
		case DIST_TO_GROUND:
			return getEntity().getDistanceToGroundText();
		case DIST_TO_WATER:
			return getEntity().getDistanceToWaterSurfaceText();
		}
		return null;
	}

}

