// $Id$
/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.media;

import org.joda.time.DateTime;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.OriginalSourceType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.mvc.element.DateElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.media.ITogglableMediaElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 16.12.2013
 *
 */
public class MediaSpecimenGeneralDetailElement extends AbstractCdmDetailElement<MediaSpecimen> implements ITogglableMediaElement{

    private final TermVocabulary mediaSpecimenVocabulary = CdmStore.getService(IVocabularyService.class).find(VocabularyEnum.MediaSpecimenKindOfUnit.getUuid());

    private TermComboElement<DefinedTerm> comboKindOfUnit;
    private LanguageStringWithLabelElement textTitleLanguageString;
    private EntitySelectionElement<AgentBase> selectionArtist;
    private DateElement date;
    private LanguageStringWithLabelElement textMethodLanguageString;
    private EntitySelectionElement<Collection> selection_collection;
    private TextWithLabelElement text_accessionNumber;
    private EntitySelectionElement<Reference> selection_publishedIn;
    private CheckboxElement checkIsPublish;

    private MediaDetailElement mediaDetailElement;
    /**
     * @param formFactory
     * @param formElement
     */
    public MediaSpecimenGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, MediaSpecimen entity, int style) {
        Media media = entity.getMediaSpecimen();
        if(media==null){
            formFactory.createLabel(formElement, "No media attached to this MediaSpecimen!");
        }
        else{
            comboKindOfUnit = formFactory.createDefinedTermComboElement(mediaSpecimenVocabulary, formElement, "Kind of Media", entity.getKindOfUnit(), style);
            textTitleLanguageString = formFactory.createLanguageStringWithLabelElement(formElement, "Motif", media.getTitle(), style);
            selectionArtist = formFactory.createSelectionElement(AgentBase.class, getConversationHolder(), formElement, "Prepared by", media.getArtist(), EntitySelectionElement.ALL, style);
            date = formFactory.createDateElement(formElement, "Preparation Date", entity.getMediaSpecimen().getMediaCreated(), style);
            textMethodLanguageString = formFactory.createLanguageStringWithLabelElement(formElement, "Method", media.getDescription(Language.getDefaultLanguage()), style);
            selection_collection = formFactory.createSelectionElement(Collection.class, getConversationHolder(), formElement, "Collection", entity.getCollection(), EntitySelectionElement.ALL, style);
            text_accessionNumber = formFactory.createTextWithLabelElement(formElement, "Accession Number", entity.getAccessionNumber(), style);

            //the first PrimaryMediaSource is used for storing the reference
            Reference<?> publishedIn = null;
            for(IdentifiableSource source:media.getSources()){
                if(source.getType()==OriginalSourceType.PrimaryMediaSource){
                    publishedIn = source.getCitation();
                    break;
                }
            }
            selection_publishedIn = formFactory.createSelectionElement(Reference.class, getConversationHolder(), formElement, "Published in", publishedIn, EntitySelectionElement.ALL, style);
            checkIsPublish = formFactory.createCheckbox(formElement, "Publish", entity.isPublish(), style);

            mediaDetailElement = formFactory.createMediaDetailElement(formElement);
            mediaDetailElement.setEntity(media);
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        Media media = getEntity().getMediaSpecimen();
        if(media==null){
            media = Media.NewInstance();
            getEntity().setMediaSpecimen(media);
        }
        if(eventSource==comboKindOfUnit){
            getEntity().setKindOfUnit(comboKindOfUnit.getSelection());
        }
        else if(eventSource==textTitleLanguageString){
            media.putTitle(textTitleLanguageString.getLanguageString());
        }
        else if(eventSource==date.getController()){
            DateTime dateTime = date.getController().getDateTime();
            media.setMediaCreated(dateTime);
        }
        else if(eventSource==selectionArtist){
            media.setArtist(selectionArtist.getSelection());
        }
        else if(eventSource==textMethodLanguageString){
            media.addDescription(textMethodLanguageString.getLanguageString());
        }
        else if (eventSource == selection_collection) {
            getEntity().setCollection(selection_collection.getSelection());
        }
        else if (eventSource == text_accessionNumber) {
            getEntity().setAccessionNumber(text_accessionNumber.getText());
        }
        else if(eventSource==selection_publishedIn){
            IdentifiableSource primaryMediaSource = null;
            for(IdentifiableSource source:media.getSources()){
                if(source.getType()==OriginalSourceType.PrimaryMediaSource){
                    primaryMediaSource = source;
                }
            }
            if(primaryMediaSource==null){
                primaryMediaSource = IdentifiableSource.NewInstance(OriginalSourceType.PrimaryMediaSource);
                media.addSource(primaryMediaSource);
            }
            else{
                primaryMediaSource.setCitation(selection_publishedIn.getSelection());
            }
        }
        else if(eventSource==checkIsPublish){
            getEntity().setPublish(checkIsPublish.getSelection());
        }
    }

    @Override
    public void toggleAdvancedMediaView() {
        if(mediaDetailElement!=null){
            mediaDetailElement.toggleAdvancedMediaView();
        }
    }

    public boolean isAdvancedMediaView() {
        if(mediaDetailElement==null){
            return false;
        }
        return mediaDetailElement.isAdvancedMediaView();
    }

}