/**
 *
 */
package eu.etaxonomy.taxeditor.ui.selection;

import java.util.EnumSet;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.cdm.persistence.hibernate.permission.CRUD;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.ILabeledElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityLabelTextProvider;

/**
 * <p>
 * Abstract AbstractSelectionElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 * @param <T>
 */
public class EntitySelectionElement<T extends CdmBase> extends
		AbstractCdmFormElement implements  SelectionListener, IEnableableFormElement, ISelectableElement, IEntityElement<T>, ILabeledElement, IConversationEnabled, Observer {

	private static final EnumSet<CRUD> UPDATE = EnumSet.of(CRUD.UPDATE);
	private static final EnumSet<CRUD> DELETE = EnumSet.of(CRUD.DELETE);
	private static final EnumSet<CRUD> CREATE = EnumSet.of(CRUD.CREATE);

	/**
	 * Bitmask for configuring functionality of selection element
	 */
	public static final int NOTHING = 0; // 000
	public static final int EDITABLE = 1 << 0; // 001
	public static final int DELETABLE = 1 << 1; // 010
	public static final int SELECTABLE = 1 << 2; // 100
	public static final int ALL = EDITABLE | DELETABLE | SELECTABLE; // 111

	protected T entity;

	protected Label label;
	protected Text text;
	protected Button button_selection;

	private SelectionArbitrator selectionArbitrator;

	protected Button button_edit;

	private final String labelString;

	private Composite selectableComposite;

	private Button button_remove;

	private final boolean isEditable;

	private final boolean isDeletable;

	private final ConversationHolder conversation;
	private Class<T> clazz;

	/**
	 * <p>
	 * Constructor for AbstractSelectionElement.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param conversation
	 *            TODO
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param labelString
	 *            a {@link java.lang.String} object.
	 * @param entity
	 *            a T object.
	 * @param isEditable
	 *            a boolean.
	 * @param isSelectable
	 *            a boolean.
	 * @param isDeletable
	 *            a boolean.
	 * @param style
	 *            a int.
	 * @param <T>
	 *            a T object.
	 */
	public EntitySelectionElement(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			String labelString, T entity, int mode, int style) {
		super(formFactory, parentElement);

		this.isEditable = (mode & EDITABLE) == EDITABLE;
		this.isDeletable = (mode & DELETABLE) == DELETABLE;
		boolean isSelectable = (mode & SELECTABLE) == SELECTABLE;

		this.labelString = (labelString == null || labelString.equals("")) ? "" : labelString;

		this.conversation = conversation;

		if (isSelectable && formFactory.getSelectionProvider() != null) {
			selectionArbitrator = formFactory.createSelectionArbitrator(this);
		}

		createControls(getLayoutComposite(), SWT.NULL);

		setEntity(entity);
	}

	public EntitySelectionElement(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, Class<T> clazz,
			String labelString, T entity, int mode, int style) {
		this(formFactory, conversation, parentElement, labelString, entity, mode, style);
		this.clazz = clazz;
	}

	private void createControls(Composite parent, int style) {

		label = formFactory.createLabel(getLayoutComposite(), labelString,
				SWT.NULL);

		addControl(label);

		selectableComposite = formFactory.createComposite(getLayoutComposite());

		int columns = 2;
		if (isEditable) {
			columns += 1;
		}
		if (isDeletable) {
			columns += 1;
		}

		selectableComposite.setLayout(LayoutConstants.LAYOUT(columns, false));
		selectableComposite.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		addControl(selectableComposite);

		text = formFactory.createText(selectableComposite, null, SWT.WRAP);
		text.setEditable(false);
		addControl(text);

		text.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
		text.setBackground(StoreUtil
				.getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND));

		button_selection = formFactory.createButton(selectableComposite, null,
				SWT.PUSH);
		button_selection.setImage(ImageResources
				.getImage(ImageResources.BROWSE_ICON));
		button_selection.setToolTipText("Browse existing");

		addControl(button_selection);
		button_selection.addSelectionListener(this);

		if (isEditable) {
			button_edit = formFactory.createButton(selectableComposite, null,
					SWT.PUSH);
			button_edit.setImage(ImageResources
					.getImage(ImageResources.EDIT_ICON));
			button_edit.setToolTipText("Edit");
			addControl(button_edit);
			button_edit.addSelectionListener(new EditListener(this));
		}

		if (isDeletable) {
			button_remove = formFactory.createButton(selectableComposite, null,
					SWT.PUSH);
			button_remove.setImage(ImageResources
					.getImage(ImageResources.TRASH_ICON));
			button_remove.setToolTipText("Remove");
			addControl(button_remove);
			button_remove.addSelectionListener(new DeleteListener(this));
		}
	}

	@Override
    public void widgetSelected(SelectionEvent e) {
		T selection = SelectionDialogFactory.getSelectionFromDialog(clazz, getShell(), getConversationHolder(), getEntity(), getParentElement());
		setSelectionInternal(selection);
	}

	/**
	 * Return the selected object
	 *
	 * @return a T object.
	 */
	public T getSelection() {
		return entity;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.IEnableableFormElement#setEnabled(boolean)
	 */
	/** {@inheritDoc} */
	@Override
	public void setEnabled(boolean enabled) {

		button_selection.setEnabled(enabled);
		if (isDeletable){
		    button_remove.setEnabled(enabled);
		}
		if (isEditable) {
			updateButtonStates();
		}

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
	    return button_selection.isEnabled();
	}

	/**
	 * <p>
	 * setSelectionInternal
	 * </p>
	 *
	 * @param selection
	 *            a T object.
	 */
	protected void setSelectionInternal(T selection) {
		if (selection != null && !selection.equals(this.entity)) {
			setEntity(selection);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
		}
	}

	/**
	 * <p>
	 * Setter for the field <code>entity</code>.
	 * </p>
	 *
	 * @param selection
	 *            a T object.
	 */
	public void setEntity(T selection) {
		this.entity = selection;
		updateElement();
	}

	/**
	 * Updates this elements view
	 */
	protected void updateElement() {
		String title = CdmUtils.Nz(getTitle());
		text.setText(title); // title can be null
		if (isEditable) {
			updateButtonStates();
		}
	}

    public void updateFromWizard() {
		updateElement();
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	/**
	 * <p>
	 * getTitle
	 * </p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	protected String getTitle() {
		if (entity != null){
			if(entity instanceof Group){
				return ((Group) entity).getName();
			} else if(entity instanceof GrantedAuthority){
				return GrantedAuthorityLabelTextProvider.getText(((GrantedAuthority) entity));
			} else if(entity instanceof User){
                return ((User) entity).getUsername();
            } else if (entity instanceof Primer){
                return ((Primer) entity).getLabel();
            } else if (entity instanceof Amplification){
                return ((Amplification) entity).getLabelCache();
            }
            else if(entity instanceof IIdentifiableEntity) {
                return ((IIdentifiableEntity) entity).getTitleCache();
            }

		}
		return "";
	}

	/** {@inheritDoc} */
	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.IEntityElement#getEntity()
	 */
	/**
	 * <p>
	 * Getter for the field <code>entity</code>.
	 * </p>
	 *
	 * @return a T object.
	 */
	@Override
	public T getEntity() {
		return entity;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.ISelectableElement#
	 * getSelectionArbitrator()
	 */
	/**
	 * <p>
	 * Getter for the field <code>selectionArbitrator</code>.
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator}
	 *         object.
	 */
	@Override
	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}

	/**
	 * Convenient access to current shell
	 *
	 * @return a {@link org.eclipse.swt.widgets.Shell} object.
	 */
    public Shell getShell() {
		return getLayoutComposite().getShell();
	}

	/** {@inheritDoc} */
	@Override
	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT
				: Resources.COLOR_TEXT_DISABLED_BACKGROUND;

		Color color = StoreUtil.getColor(colorId);
		text.setBackground(color);
	}

	private class DeleteListener extends SelectionAdapter {

		private final EntitySelectionElement<T> selectionElement;

		public DeleteListener(EntitySelectionElement<T> selectionElement) {
			this.selectionElement = selectionElement;
		}

		@Override
		public void widgetSelected(SelectionEvent e) {
			setEntity(null);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(
					selectionElement, null));
		}
	}

	private class EditListener extends SelectionAdapter {

        private final EntitySelectionElement<T> selectionElement;

		public EditListener(EntitySelectionElement<T> selectionElement) {
			this.selectionElement = selectionElement;
		}

		/** {@inheritDoc} */
		@Override
		public void widgetSelected(SelectionEvent e) {

	        IService<T> service = CdmStore.getService(entity);
	        T loadedEntity = service.load(entity.getUuid());
            if (loadedEntity==null){
	            MessageDialog.openInformation(null,"Save changes", "You have made changes that must be saved before this query can be executed");
	            return;
	        }
			WizardDialog dialog = new WizardDialog(selectionElement.getShell(),
					new EditFromSelectionWizard(selectionElement));
			if (dialog.open() == IStatus.OK) {

				selectionElement.updateFromWizard();
			}
		}
	}

	// not used
	/** {@inheritDoc} */
	@Override
	public void widgetDefaultSelected(SelectionEvent e) {
	}

	/**
	 * <p>
	 * getConversationHolder
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
	 *         object.
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
	    if(!label.isDisposed()){
	        label.setBackground(color);
	    }
	}

	/** {@inheritDoc} */
	@Override
	public void setLabel(String labelString) {
		if (label != null) {
			label.setText(labelString);
		}
	}

	/**
	 * <p>
	 * Getter for the field <code>label</code>.
	 * </p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
	public String getLabel() {
		if (label != null) {
			return label.getText() + " : ";
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void update(CdmDataChangeMap changeEvents) {
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement#removeElements()
	 */
	@Override
    public void removeElements(){
		super.removeElements();
		LoginManager loginManager = CdmStore.getLoginManager();
		loginManager.addObserver(this);
	}

	@Override
	public void update(Observable o, Object arg) {
		if(o instanceof LoginManager){
			updateButtonStates();
		}
	}

	private void updateButtonStates() {
	    if(button_edit != null && !button_selection.isDisposed()){
	        button_edit.setEnabled(isEditable && button_selection.isEnabled() && getEntity() != null  && CdmStore.currentAuthentiationHasPermission(getEntity(), UPDATE));
	    }
	}
}
